/*
 * VocabularyTree.java     1.0     2001/12/27
 *
 * Copyright (c) 2001 Andreas Dangel <a.dangel@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package editor;

import libraries.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;

public class VocabularyTree extends JTree {

    private String filename;
    private Vocabulary vocabulary;

    EditorEditLessonDialog editLessonDialog;
    EditorEditVocabularyDialog editVocabularyDialog;
    EditorEditVocableDialog editVocableDialog;

    VocabularyTree(String file) {
        super();

        filename = file;
        vocabulary = new Vocabulary();
        vocabulary.loadFromFile(filename);

        setModel(vocabulary.getTreeModel());

        editLessonDialog = null;
        editVocabularyDialog = null;
        editVocableDialog = null;
    }

    public void saveToFile() {
        saveToFile(filename);
        Editor.setSaved(true);
    }

    public void saveToFile(String file) {
        vocabulary.saveToFile(file);
    }

    public String getFileName() {
        return filename;
    }

    public void editSelection() {
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /* 3 mglichkeiten:
         * selection instanceOf Vocabulary
         * selection instanceof Lesson
         * selection instanceof Vocable
         */
        if (selection instanceof Vocabulary) {
            editVocabularyDialog = new EditorEditVocabularyDialog((Vocabulary) selection);
        } else if (selection instanceof Lesson) {
            editLessonDialog = new EditorEditLessonDialog((Lesson) selection);
        } else if (selection instanceof Vocable) {
            editVocableDialog = new EditorEditVocableDialog((Vocable) selection);
        }
    }

    public void addLesson() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            selectionNode = (DefaultMutableTreeNode) getModel().getRoot();
            selection = selectionNode.getUserObject();
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /* 3 mglichkeiten:
         * selection instanceOf Vocabulary
         * selection instanceof Lesson
         * selection instanceof Vocable
         */
        Lesson lessonToAdd = new Lesson();

        if (selection instanceof Vocabulary) {
            vocabulary.add(lessonToAdd);
        } else if (selection instanceof Lesson) {
            ((Lesson) selection).lessonsAdd(lessonToAdd);
        } else if (selection instanceof Vocable) {
            vocabulary.find((Vocable) selection).lessonsAdd(lessonToAdd);
        }

        setModel(vocabulary.getTreeModel());

        clearSelection();
        addSelectionPath(getPathToLesson(lessonToAdd));
        editSelection();
    }

    public void addVocable() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /* 3 mglichkeiten:
         * selection instanceOf Vocabulary
         * selection instanceof Lesson
         * selection instanceof Vocable
         */
        Vocable vocableToAdd = new Vocable();

        if (selection instanceof Vocabulary) {
            return;
        } else if (selection instanceof Lesson) {
            ((Lesson) selection).add(vocableToAdd);
        } else if (selection instanceof Vocable) {
            vocabulary.find((Vocable) selection).add(vocableToAdd);
        }

        setModel(vocabulary.getTreeModel());

        clearSelection();
        addSelectionPath(getPathToVocable(vocableToAdd));
        editSelection();
    }

    public TreePath getPathToLesson(Lesson les) {
        DefaultMutableTreeNode node = les.getTreeNode();
        TreePath path = new TreePath(node.getPath());
        return path;
    }

    public TreePath getPathToVocable(Vocable voc) {
        DefaultMutableTreeNode node = voc.getTreeNode();
        TreePath path = new TreePath(node.getPath());
        return path;
    }

    public void repaintModel() {
        setModel(vocabulary.getTreeModel());
    }

    public void removeSelection() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /* 3 mglichkeiten:
         * selection instanceOf Vocabulary
         * selection instanceof Lesson
         * selection instanceof Vocable
         */
        Object parent = null;
        if (selection instanceof Vocabulary) {
            vocabulary.initialize();
        } else if (selection instanceof Lesson) {
            parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();
            if (parent instanceof Vocabulary) {
                vocabulary.remove((Lesson) selection);
            } else if (parent instanceof Lesson) {
                ((Lesson) parent).lessonsRemove((Lesson) selection);
            }
        } else if (selection instanceof Vocable) {
            parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();
            if (parent instanceof Lesson) {
                ((Lesson) parent).remove((Vocable) selection);
            }
        }

        setModel(vocabulary.getTreeModel());
        Editor.setSaved(false);

        if (parent != null) {
            if (parent instanceof Lesson) {
                clearSelection();
                addSelectionPath(getPathToLesson((Lesson) parent));
            }
        }
    }

    public void moveSelectionUp() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /*
         * 3 Mglichkeiten fr selection:
         * instanceof Vocabulary -> nicht tun
         * instanceof Lesson
         * instanceof Vocable
         */
        if (selection instanceof Vocabulary) {
            return;
        }

        Object parent;
        parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();

        if (selection instanceof Lesson) {
            int index;

            if (parent instanceof Vocabulary) {
                Vocabulary parentVoc = (Vocabulary) parent;
                index = parentVoc.indexOf((Lesson) selection);

                if (index == 0) {
                } else {
                    parentVoc.exchange(index, index - 1);
                    setModel(vocabulary.getTreeModel());
                    clearSelection();
                    addSelectionPath(getPathToLesson((Lesson) selection));
                    Editor.setSaved(false);
                }
            } else if (parent instanceof Lesson) {
                Lesson parentLesson = (Lesson) parent;
                index = parentLesson.lessonsIndexOf((Lesson) selection);

                if (index == 0) {
                    moveSelectionLeft();
                } else {
                    parentLesson.lessonsExchange(index, index - 1);
                    setModel(vocabulary.getTreeModel());
                    clearSelection();
                    addSelectionPath(getPathToLesson((Lesson) selection));
                    Editor.setSaved(false);
                }
            }


        } else if (selection instanceof Vocable) {
            /* Plan:
             * - index innerhalb der bergeordneten Lektion bestimmen
             * - exchange(index, index-1)
             * - wenn index = 0, dann moveSelectionLeft()
             */
            int index;
            Lesson parentLesson;

            parentLesson = (Lesson) parent;
            index = parentLesson.indexOf((Vocable) selection);

            if (index == 0) {
                moveSelectionLeft();
            } else {
                parentLesson.exchange(index, index - 1);
                setModel(vocabulary.getTreeModel());
                clearSelection();
                addSelectionPath(getPathToVocable((Vocable) selection));
                Editor.setSaved(false);
            }
        }

    }

    public void moveSelectionDown() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /*
         * 3 Mglichkeiten fr selection:
         * instanceof Vocabulary -> nicht tun
         * instanceof Lesson
         * instanceof Vocable
         */
        if (selection instanceof Vocabulary) {
            return;
        }

        Object parent;
        parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();

        if (selection instanceof Lesson) {
            int index;

            if (parent instanceof Vocabulary) {
                Vocabulary parentVoc = (Vocabulary) parent;
                index = parentVoc.indexOf((Lesson) selection);

                if (index == (parentVoc.size() - 1)) {
                } else {
                    parentVoc.exchange(index, index + 1);
                    setModel(vocabulary.getTreeModel());
                    clearSelection();
                    addSelectionPath(getPathToLesson((Lesson) selection));
                    Editor.setSaved(false);
                }
            } else if (parent instanceof Lesson) {
                Lesson parentLesson = (Lesson) parent;
                index = parentLesson.lessonsIndexOf((Lesson) selection);

                if (index == (parentLesson.lessonsSize() - 1)) {
                    moveSelectionLeft();
                } else {
                    parentLesson.lessonsExchange(index, index + 1);
                    setModel(vocabulary.getTreeModel());
                    clearSelection();
                    addSelectionPath(getPathToLesson((Lesson) selection));
                    Editor.setSaved(false);
                }
            }


        } else if (selection instanceof Vocable) {
            /* Plan:
             * - index innerhalb der bergeordneten Lektion bestimmen
             * - exchange(index, index+1)
             * - wenn index = 0, dann moveSelectionLeft()
             */
            int index;
            Lesson parentLesson;

            parentLesson = (Lesson) parent;
            index = parentLesson.indexOf((Vocable) selection);

            if (index == (parentLesson.lessonsSize() - 1)) {
                moveSelectionLeft();
            } else {
                parentLesson.exchange(index, index + 1);
                setModel(vocabulary.getTreeModel());
                clearSelection();
                addSelectionPath(getPathToVocable((Vocable) selection));
                Editor.setSaved(false);
            }
        }
    }

    public void moveSelectionLeft() {
        // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /*
         * 3 Mglichkeiten fr selection:
         * instanceof Vocabulary -> nicht tun
         * instanceof Lesson
         * instanceof Vocable
         */
        if (selection instanceof Vocabulary) {
            return;
        }

        Object parent, parentParent;
        parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();
        if (parent instanceof Vocabulary) {
            moveSelectionUp();
            return;
        } else {
            parentParent = ((DefaultMutableTreeNode) ((DefaultMutableTreeNode) selectionNode.getParent()).getParent()).getUserObject();
        }

        if (selection instanceof Lesson) {
            if (parent instanceof Lesson) {
                if (parentParent instanceof Vocabulary) {
                    Vocabulary parentParentVoc = (Vocabulary) parentParent;
                    Lesson parentLesson = (Lesson) parent;
                    int index = parentParentVoc.indexOf(parentLesson);
                    parentLesson.lessonsRemove((Lesson) selection);
                    parentParentVoc.add(index + 1, (Lesson) selection);
                } else if (parentParent instanceof Lesson) {
                    Lesson parentParentLesson = (Lesson) parentParent;
                    Lesson parentLesson = (Lesson) parent;
                    int index = parentParentLesson.lessonsIndexOf(parentLesson);
                    parentLesson.lessonsRemove((Lesson) selection);
                    parentParentLesson.lessonsAdd(index + 1, (Lesson) selection);
                }
            }
        } else if (selection instanceof Vocable) {
            if (parent instanceof Lesson) {
                if (parentParent instanceof Vocabulary) {
                    return;
                } else if (parentParent instanceof Lesson) {
                    Lesson parentParentLesson = (Lesson) parentParent;
                    Lesson parentLesson = (Lesson) parent;
                    parentLesson.remove((Vocable) selection);
                    parentParentLesson.add((Vocable) selection);
                }
            }
        }

        setModel(vocabulary.getTreeModel());
        Editor.setSaved(false);
        clearSelection();
        if (selection instanceof Lesson) {
            addSelectionPath(getPathToLesson((Lesson) selection));
        } else if (selection instanceof Vocable) {
            addSelectionPath(getPathToVocable((Vocable) selection));
        }

    }

    public void moveSelectionRight() {
         // was wurde markiert?
        TreePath selectionPath = getSelectionPath();
        DefaultMutableTreeNode selectionNode = null;
        Object selection = null;

        /* nichts markiert */
        if (selectionPath == null) {
            return;
        } else {
            selectionNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            selection = selectionNode.getUserObject();
        }

        /*
         * 3 Mglichkeiten fr selection:
         * instanceof Vocabulary -> nicht tun
         * instanceof Lesson
         * instanceof Vocable
         */
        if (selection instanceof Vocabulary) {
            return;
        }

        Object parent, parentParent;
        parent = ((DefaultMutableTreeNode) selectionNode.getParent()).getUserObject();
        if (!(parent instanceof Vocabulary)) {
            parentParent = ((DefaultMutableTreeNode) ((DefaultMutableTreeNode) selectionNode.getParent()).getParent()).getUserObject();
        }

        if (selection instanceof Lesson) {
            if (parent instanceof Vocabulary) {
                Vocabulary parentVoc = (Vocabulary) parent;
                int index = parentVoc.indexOf((Lesson) selection);
                if (index == 0) {
                    return;
                }
                parentVoc.remove(index);

                Lesson lesBefore = parentVoc.get(index - 1);
                lesBefore.lessonsAdd((Lesson) selection);
            } else if (parent instanceof Lesson) {
                Lesson parentLesson = (Lesson) parent;
                int index = parentLesson.lessonsIndexOf((Lesson) selection);
                if (index == 0) {
                    return;
                }
                parentLesson.lessonsRemove(index);

                Lesson lesBefore = parentLesson.lessonsGet(index - 1);
                lesBefore.lessonsAdd((Lesson) selection);
            }
        } else if (selection instanceof Vocable) {
            return;
        }

        setModel(vocabulary.getTreeModel());
        clearSelection();
        addSelectionPath(getPathToLesson((Lesson) selection));
        Editor.setSaved(false);
    }
}