/*
 * Lesson.java     1.0    2001/12/27
 *
 * Copyright (c) 2001 Andreas Dangel <a.dangel@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package libraries;

import java.util.Vector;
import javax.swing.tree.*;

/**
 * Lesson reprsentiert eine komplette Lektion mit vielen Vokabeln.
 *
 * @see Vocable
 *
 * @version 1.0 2001/12/27
 * @author  Andreas Dangel <a.dangel@gmx.de>
 */
public class Lesson {

    private final static String INDENT = Vocabulary.INDENT;

    private Vector vocables;
    private Vector lessons;
    private String title;

    private DefaultMutableTreeNode treeNode;


    /**
     * ruft initialize() auf.
     *
     * @see Lesson#initialize
     */
    public Lesson() {
        initialize();
    }

    /**
     * Initialisiert das Objekt.
     * Alle Daten werden auf null gesetzt.
     */
    public void initialize() {
        title = "";
        vocables = new Vector();
        lessons = new Vector();

        createTreeNode();
    }

    /**
     * Erzeugt ein DefaultMutableTreeNode-Objekt.
     *
     * @return TreeNode-Objekt.
     */
    public DefaultMutableTreeNode createTreeNode() {
        treeNode = new DefaultMutableTreeNode(this, true);

        /* Lektionen */
        for (int i = 0; i < this.lessonsSize(); i++) {
            treeNode.add(this.lessonsGet(i).createTreeNode());
        }

        /* Vokabeln */
        for (int i = 0; i < this.size(); i++) {
            treeNode.add(this.get(i).createTreeNode());
        }

        return treeNode;
    }

    /**
     * Setzt den Titel der Lektion.
     *
     * @param s der Titel
     */
    public void setTitle(String s) {
        title = s;
    }

    /**
     * Gibt den Titel zurck.
     *
     * @return Titel
     */
    public String getTitle() {
        return title;
    }


    /**
     * Fgt eine neue Vokabel am Ende hinzu.
     *
     * @param voc die Vokabel
     */
    public void add(Vocable voc) {
        vocables.add(voc);
    }

    /**
     * Fgt eine neue Lektion hinzu.
     *
     * @param les die Lektion
     */
    public void lessonsAdd(Lesson les) {
        lessons.add(les);
    }

    /**
     * Fgt eine neue Vokabel der gegebenen Position ein.
     *
     * @param index Position
     * @param voc   die Vokabel
     */
    public void add(int index, Vocable voc) {
        vocables.add(index, voc);
    }


    /**
     * Fgt eine neue Lektion an der gegebenen Position ein.
     *
     * @param index Position
     * @param les   die Lektion
     */
    public void lessonsAdd(int index, Lesson les) {
        lessons.add(index, les);
    }

    /**
     * Lscht alle Vokabeln.
     */
    public void clear() {
        vocables.clear();
    }

    /**
     * Lscht alle Lektionen.
     */
    public void lessonsClear() {
        lessons.clear();
    }

    /**
     * Lscht die Vokabel an der gegebenen Position.
     *
     * @param index Position
     * @return die gelschte Vokabel
     */
    public Vocable remove(int index) {
        return (Vocable) vocables.remove(index);
    }

    /**
     * Lscht die Lektion an der gegebenen Position.
     *
     * @param index Position
     * @return die gelschte Lektion
     */
    public Lesson lessonsRemove(int index) {
        return (Lesson) lessons.remove(index);
    }

    /**
     * Lscht die gegebene Vokabel aus der Lektion.
     *
     * @param voc die Vokabel, die gelscht werden soll
     * @return true, wenn Vokabel gefunden und gelscht wurde
     */
    public boolean remove(Vocable voc) {
        int index = this.indexOf(voc);
        if (index > -1) {
            this.remove(index);
            return true;
        }
        return false;
    }


    /**
     * Lscht die gegebene Lektion.
     *
     * @param les die Lektion, die gelscht werden soll
     * @return true, wenn die Lektion gefunden und gelscht wurde
     */
    public boolean lessonsRemove(Lesson les) {
        int index = this.lessonsIndexOf(les);
        if (index > -1) {
            this.lessonsRemove(index);
            return true;
        }
        return false;
    }

    /**
     * Vertauscht zwei Vokabeln.
     *
     * @param index1 erste Vokabel
     * @param index2 zweite Vokabel
     */
    public void exchange(int index1, int index2) {
        if (index1 == index2) {
            // do nothing
        } else if (index1 < index2) {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 - 1));
        } else {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 + 1));
        }
    }

    /**
     * Vertauscht zwei Lektionen.
     *
     * @param index1 erste Lektion
     * @param index2 zweite Lektion
     */
    public void lessonsExchange(int index1, int index2) {
        if (index1 == index2) {
            // do nothing
        } else if (index1 < index2) {
            this.lessonsAdd(index2, this.lessonsRemove(index1));
            this.lessonsAdd(index1, this.lessonsRemove(index2 - 1));
        } else {
            this.lessonsAdd(index2, this.lessonsRemove(index1));
            this.lessonsAdd(index1, this.lessonsRemove(index2 + 1));
        }
    }

    /**
     * Gibt den Index der Vokabel zurck.
     *
     * @param voc die gesuchte Vokabel
     * @return index oder -1, wenn Vokabel nicht vorhanden
     */
    public int indexOf(Vocable voc) {
        return vocables.indexOf(voc);
    }

    /**
     * Gibt den Index der Lektion zurck.
     *
     * @param les die gesuchte Lektion
     * @return index oder -1, wenn Lektion nicht vorhanden
     */
    public int lessonsIndexOf(Lesson les) {
        return lessons.indexOf(les);
    }

    /**
     * Gibt die Vokabel an der gegebenen Position zurck.
     *
     * @param i Position
     * @return die Vokabel
     */
    public Vocable get(int i) {
        return (Vocable) vocables.get(i);
    }

    /**
     * Gibt die Lektion an der gegebenen Position zurck.
     *
     * @param i Position
     * @return die Lektion
     */
    public Lesson lessonsGet(int i) {
        return (Lesson) lessons.get(i);
    }

    /**
     * Gibt die Anzahl der Vokabeln zurck.
     *
     * @return Anzahl
     */
    public int size() {
        return vocables.size();
    }

    /**
     * Gibt die Anzahl der Lektionen zurck.
     *
     * @return Anzahl
     */
    public int lessonsSize() {
        return lessons.size();
    }


    /**
     * Fgt eine komplette Lektion hinzu.
     *
     * @param les die Lektion, die hinzugefgt werden soll.
     */
    public void add(Lesson les) {
        for (int i = 0; i < les.size(); i++) {
            this.add(les.get(i));
        }
    }

    /**
     * Erstellt einen XML-Code.
     *
     * @return StringList mit XML-Code.
     * @see Lesson#parseXML(String)
     */
    public StringList toXML() {
        StringList sl = new StringList();
        sl.add(INDENT + "<Lesson>");
        sl.add(INDENT + INDENT + "<TitleOfLesson>" + this.getTitle() + "</TitleOfLesson>");

        /* untergeordnete Lektionen: */
        for (int i = 0; i < this.lessonsSize(); i++) {
            sl.add(this.lessonsGet(i).toXML());
        }

        /* Vokabeln */
        for (int i = 0; i < this.size(); i++) {
            sl.add(this.get(i).toXML());
        }

        sl.add(INDENT + "</Lesson>");

        return sl;
    }

    /**
     * Liest die Daten aus einem XML-Code aus.
     * Initialisert zuvor das Objekt neu.
     *
     * @param s String mit XML-Code.
     * @see Lesson#toXML
     */
    public void parseXML(String s) {
        int index;
        int length;
        String stemp;
        Vocable voc;
        Lesson les;

        initialize();

        //* Bis zum ersten "<Lesson>" lschen (inklusiv)
        index = s.indexOf("<Lesson>");
        length = "<Lesson>".length();
        s = s.substring(index + length);

        //* Bis zum nchsten "<TitleOfLesson>" lschen (inklusiv)
        index = s.indexOf("<TitleOfLesson>");
        length = "<TitleOfLesson>".length();
        s = s.substring(index + length);

        //* Bis zum nchsten "</TitleOfLesson>" = title (exklusiv)
        index = s.indexOf("</TitleOfLesson>");
        stemp = s.substring(0, index);
        this.setTitle(stemp);
        length = "</TitleOfLesson>".length();
        s = s.substring(index + length);

        /* untergeordnete Lektionen? */

        /* wiederhole solange wie <Lesson> vorhanden ist */
        while (s.indexOf("<Lesson>") > -1) {
            /* Tiefe/Stufe/... = 1 */
            int depth = 1;

            /* sichere index */
            index = s.indexOf("<Lesson>");

            /* aktuelle Position */
            int position = index;

            /* wiederhole solange depth > 0 */
            while (depth > 0) {
                if ((s.indexOf("<Lesson>", position + 1) > -1)
                        && (s.indexOf("<Lesson>", position + 1) < s.indexOf("</Lesson>", position + 1))) {
                    /* nchstes Element: <Lesson> */
                    depth++;

                    /* aktuelle Position */
                    position = s.indexOf("<Lesson>", position + 1);
                } else {
                    /* nchstes Element msste </Lesson> sein */
                    depth--;

                    /* aktuelle Position */
                    position = s.indexOf("</Lesson>", position + 1);
                }
            }

            /* jetzt msste depth == 0 sein */
            length = "</Lesson>".length();
            stemp = s.substring(index, position + length);
            s = s.substring(position + length);

            /* neue Lektion */
            les = new Lesson();
            les.parseXML(stemp);
            this.lessonsAdd(les);
        }

        /* Vokabeln: */
        while ((s.indexOf("<Vocable>") != -1) && (s.indexOf("</Vocable>") < s.indexOf("</Lesson>"))) {
            //* Bis zum nchsten "<Vocable>" lschen (exklusiv)
            index = s.indexOf("<Vocable>");
            s = s.substring(index);

            //* Bis zum nchsten "</Vocable>" = new Vocable (inklusiv)
            index = s.indexOf("</Vocable>");
            length = "</Vocable>".length();
            stemp = s.substring(0, index + length);
            s = s.substring(index + length);

            voc = new Vocable();
            voc.parseXML(stemp);
            this.add(voc);
        }
    }

    /**
     * Liest die Daten aus einem XML-Code aus.
     *
     * @param sl StringList mit XML-Code.
     * @see Lesson#parseXML(String)
     */
    public void parseXML(StringList sl) {
        parseXML(sl.getText());
    }

    /**
     * Erstellt HTML-Code (exportiert die Daten).
     *
     * @return StringList mit XML-Code.
     */
    public StringList toHTML() {
        StringList sl = new StringList();

        sl.add("<h2>" + this.getTitle() + "</h2>");

        /* Vokabeln: */
        if (this.size() > 0) {
            sl.add("<table border=\"1\">");
            sl.add(INDENT + "<tr>");
            sl.add(INDENT + INDENT + "<th>Word</th>");
            sl.add(INDENT + INDENT + "<th>Translations</th>");
            sl.add(INDENT + INDENT + "<th>Remark</th>");
            sl.add(INDENT + "</tr>");

            for (int i = 0; i < this.size(); i++) {
                sl.add(this.get(i).toHTML());
            }

            sl.add("</table>");
        }

        /* untergeordnete Lektionen: */
        for (int i = 0; i < this.lessonsSize(); i++) {
            sl.add(this.lessonsGet(i).toHTML());
        }

        sl.add("");
        return sl;
    }

    /**
     * Gibt die Daten im Textformat aus (exportiert die Daten).
     *
     * @return StringList mit Textdaten
     */
    public StringList toText() {
        StringList sl = new StringList();
        String stemp = "";
        String stemp2 = "Lesson: " + this.getTitle();
        sl.add(stemp2);

        for (int i = 0; i < stemp2.length(); i++) {
            stemp = stemp + "-";
        }
        sl.add(stemp);

        /* Vokabeln: */
        for (int i = 0; i < this.size(); i++) {
            sl.add(this.get(i).toText());
        }
        sl.add("");

        /* untergeordnete Lektionen */
        for (int i = 0; i < this.lessonsSize(); i++) {
            sl.add(this.lessonsGet(i).toText());
        }

        return sl;
    }

    /**
     * Gibt den Titel der Lektion als (sogar formatiert, mit HTML) String aus.
     * Wichtig fr JTree...
     */
    public String toString() {
        String s;
        s = "<html>";
        s = s + "<b>" + this.getTitle() + "</b>";
        s = s + "</html>";

        return s;
    }

    /**
     * Sucht eine Vokabel in der Lektion. Das Suchergebnis ist
     * wieder in einer Lektion (lesson).
     *
     * @param voc eine Vokabel mit bestimmten Eigenschaften
     * @return eine Lektion mit Suchergebnissen
     */
    public Lesson search(Vocable voc) {
        Lesson les = new Lesson();
        Vocable vtemp;
        les.setTitle("Result");

        /* Vokabeln: */
        for (int i = 0; i < this.size(); i++) {
            vtemp = this.get(i);

            if ((voc.getWord().equals("") == false) && (vtemp.getWord().equals(voc.getWord()))) {
                les.add(vtemp);
            } else if ((voc.getTag().equals("") == false) && (vtemp.getTag().equals(voc.getTag()))) {
                les.add(vtemp);
            } else if ((voc.getTranslations().size() > 0) && (vtemp.getTranslations().equals(voc.getTranslations()))) {
                les.add(vtemp);
            } else if ((voc.getRemark().equals("") == false) && (vtemp.getRemark().equals(voc.getRemark()))) {
                les.add(vtemp);
            }
        }

        /* untergeordnete Lektionen: */
        for (int i = 0; i < this.lessonsSize(); i++) {
            les.add(this.lessonsGet(i).search(voc));
        }

        return les;
    }

    /**
     * Sucht zum Titel die Lektion.
     *
     * @return die Lektion oder null wenn nichts gefunden wurde.
     */
    public Lesson searchLesson(String title) {
        Lesson ret = null;

	// untergeordnete Lektionen durchsuchen...
	for (int i = 0; i < this.lessonsSize(); i++) {
	    if (this.lessonsGet(i).getTitle().equals(title)) {
	        ret = this.lessonsGet(i);
		return ret;
	    }
	    // die untergeordneten Lektionen der untergeordneten durchsuchen...
	    ret = this.lessonsGet(i).searchLesson(title);
	    if (ret != null) {
	        return ret;
	    }
	}
    
        return null;
    }


    /**
     * Gibt ein DefaultMutableTreeNode-Objekt zurck, das fr das JTree-Objekt
     * verwendet werden kann.
     *
     * @see javax.swing.tree.DefaultMutableTreeNode
     * @see javax.swing.JTree
     *
     * @return ein fertiges DefaultMutableTreeNode-Objekt.
     */
    public DefaultMutableTreeNode getTreeNode() {
        return treeNode;
    }

    /**
     * Findet die Lektion zu einer Vokabel.
     *
     * @param voc die gesuchte Vokabel
     * @return die dazugehrende Lektion oder null, wenn keine gefunden wurde.
     */
    public Lesson find(Vocable voc) {
        int index = -1;
        Lesson les = null;

        /* Lektionen */
        for (int i = 0; i < this.lessonsSize(); i++) {
            les = this.lessonsGet(i).find(voc);
            if (les != null) {
                return les;
            }
        }

        /* Vokabeln */
        index = this.indexOf(voc);
        if (index > -1) {
            return this;
        }

        return null;
    }
}
