/*
 * StringList.java     1.0     2001/12/27
 *
 * Copyright (c) 2001 Andreas Dangel <a.dangel@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package libraries;

import java.io.*;
import java.util.Vector;


/**
 * Stringlist reprsentiert eine variable Liste mit Strings. Es ist intern
 * durch java.util.Vektor realisiert und bietet hnliche Methoden an.
 *
 * @see java.util.Vector
 *
 * @version 1.0 2001/12/27
 * @author  Andreas Dangel <a.dangel@gmx.de>
 */
public class StringList {

    /**
     * Das Zeilenende-Zeichen.
     * @see java.io.BufferedReader
     */
    public static final String NEWLINE = "\r\n"; // Zeilenumbruch wie in DOS

    private Vector strings;


    /**
     * ein leerer Konstruktor. Ruft initialize() auf.
     */
    public StringList() {
        initialize();
    }

    /**
     * Initialisiert das StringList-Objekt. Lscht alle vorhandenen Daten.
     */
    public void initialize() {
        strings = new Vector();
    }


    /**
     * Fgt ein String am Ende der Liste hinzu.
     *
     * @param s der hinzuzufgende String.
     */
    public void add(String s) {
        strings.add(s);
    }

    /**
     * Fgt ein String an einer bestimmten Position in der Liste ein.
     *
     * @param index die Position
     * @param s     der hinzuzufgende String
     */
    public void add(int index, String s) {
        strings.add(index, s);
    }


    /**
     * Leert die Liste.
     */
    public void clear() {
        strings.clear();
    }


    /**
     * Lscht einen bestimmten String aus der Liste und gibt diesen zurck.
     *
     * @param index Position des zu lschenden Strings.
     * @return den gelschten String
     */
    public String remove(int index) {
        return (String) strings.remove(index);
    }

    /**
     * Lscht einen bestimmten String, falls er vorhanden ist.
     *
     * @param s den zu lschenden String
     * @return true, wenn der String gefunden und gelscht wurde.
     */
    public boolean remove(String s) {
        for (int i = 0; i < this.size(); i++) {
            if (this.get(i).equals(s)) {
                this.remove(i);
                return true;
            }
        }
        return false;
    }


    /**
     * Vertauscht zwei Eintrge.
     *
     * @param index1 der erste String
     * @param index2 der zweite String
     */
    public void exchange(int index1, int index2) {
        if (index1 == index2) {
            // do nothing
        } else if (index1 < index2) {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 - 1));
        } else {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 + 1));
        }
    }

    /**
     * Gibt den Index des ersten Vorkommens von einem String aus.
     *
     * @param s der gesuchte String
     * @return der Index oder -1, wenn der String nicht gefunden wurde.
     */
    public int indexOf(String s) {
        return strings.indexOf(s);
    }

    /**
     * Gibt den String von einer bestimmten Position zurck.
     *
     * @param index die Position
     * @return der String
     */
    public String get(int index) {
        return (String) strings.get(index);
    }

    /**
     * Ersetzt den String an der angegebenen Position durch den
     * angegebenen String.
     *
     * @param index Position
     * @param s     String
     * @return den ersetzen String
     */
    public String set(int index, String s) {
        this.add(index, s);
        return this.remove(index + 1);
    }

    /**
     * Gibt die Anzahl der Eintrge zurck.
     *
     * @return int
     */
    public int size() {
        return strings.size();
    }

    /**
     * Fgt eine komplette Stringliste hinzu.
     *
     * @param sl die hinzuzufgenden StringList
     */
    public void add(StringList sl) {
        for (int i = 0; i < sl.size(); i++) {
            this.add(sl.get(i));
        }
    }

    /**
     * Prft, ob mindestens ein String bereinstimmt.
     *
     * @param sl die zu vergleichende StringList.
     * @return true, wenn mindestens ein String bereinstimmt.
     */
    public boolean equals(StringList sl) {
        for (int i = 0; i < this.size(); i++) {
            for (int j = 0; j < sl.size(); j++) {
                if (this.get(i).equals(sl.get(j))) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Gibt den Inhalt der Liste als einen einzigen String zurck.
     *
     * @return String
     *
     * @see StringList#setText(String)
     */
    public String getText() {
        String s = "";
        for (int i = 0; i < this.size(); i++) {
            s = s + this.get(i) + NEWLINE;
        }
        return s;
    }

    /**
     * Spaltet den String in Zeilen auf. Die StringList wird davor neu
     * initialisiert.
     *
     * @param s der String
     *
     * @see StringList#NEWLINE
     * @see StringList#getText()
     * @see java.io.BufferedReader
     * @see java.io.StreamReader
     */
    public void setText(String s) {
        BufferedReader in = null;
        try {
            in = new BufferedReader(new StringReader(s));
            String line;

            this.initialize();

            while((line = in.readLine()) != null) {
                this.add(line);
            }
        } catch (IOException e) {
            System.err.println("in StringList.setText(String):\n" + e);
        } finally {
            try {
                in.close();
            } catch (Exception e1) {
                System.err.println("in StringList.setText(String):\n" + e1);
            }
        }
    }


    /**
     * Ldt eine Datei in die StringList. Die Liste wird davor
     * neu initialisiert.
     *
     * @param filename der Dateiname
     * @see StringList#saveToFile(String)
     */
    public void loadFromFile(String filename) {
        BufferedReader in = null;

        try {
	    File file = new File(filename);
	    file.createNewFile();
	    file = null;

            in = new BufferedReader(new FileReader(filename));
            String line;

            this.initialize();

            while ((line = in.readLine()) != null) {
                this.add(line);
            }
        } catch (IOException e) {
            System.err.println("in StringList.loadFromFile(String):\n" + e);
	    System.exit(1);
        } finally {
            try {
                in.close();
            } catch (IOException e1) {
                System.err.println("in StringList.loadFromFile(String):\n" + e1);
            }
        }
    }

    /**
     * Speichert die Stringliste in eine Datei.
     *
     * @param filename der Dateiname
     * @see StringList#loadFromFile(String)
     */
    public void saveToFile(String filename) {
        PrintWriter out = null;

        try {
            out = new PrintWriter(new FileWriter(filename));

            for (int i = 0; i < this.size(); i++) {
                out.println(this.get(i));
            }
        } catch (IOException e) {
            System.err.println("in StringList.saveToFile(String):\n" + e);
        } finally {
            out.close();
        }
    }
}
