/*
 * Vocable.java     1.0    2001/12/27
 *
 * Copyright (c) 2001 Andreas Dangel <a.dangel@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package libraries;

import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Vocable reprsentiert eine einzige Vokabel.
 *
 * @version 1.0 2001/12/27
 * @author  Andreas Dangel <a.dangel@gmx.de>
 */
public class Vocable {

    /** Substantiv */
    public static final int TYPE_NOUN = 0;
    /** Verb */
    public static final int TYPE_VERB = 1;
    /** Adjektiv */
    public static final int TYPE_ADJECTIVE = 2;
    /** Pronomen */
    public static final int TYPE_PRONOUN = 3;
    /** Unvernderlich */
    public static final int TYPE_INVARIABLE = 4;
    /** sonstiges */
    public static final int TYPE_OTHER = 5;

    /**
     * Mit diesem Array lassen sich die Integer-Konstanten
     * einfach in einen String umwandeln
     */
    public static final String[] TYPES = { "noun", "verb", "adjective",
            "pronoun", "invariable", "other" };


    private static final String INDENT = Vocabulary.INDENT;

    private String word;
    private int type;
    private String tag;
    private StringList translations;
    private String remark;

    private DefaultMutableTreeNode treeNode;

    /**
     * Ruft initialize() auf.
     *
     * @see Vocable#initialize
     */
    public Vocable() {
        initialize();
    }

    /**
     * Initialisiert das Objekt.
     */
    public void initialize() {
        word = "";
        type = TYPE_OTHER;
        tag = "";
        translations = new StringList();
        remark = "";

        createTreeNode();
    }

    /**
     * Erzeugt ein DefaultMutableTreeNode-Objekt.
     *
     * @return TreeNode-Objekt.
     */
    public DefaultMutableTreeNode createTreeNode() {
        treeNode = new DefaultMutableTreeNode(this, false);
        return treeNode;
    }

    /**
     * Setzt das Wort
     *
     * @param s das Wort
     */
    public void setWord(String s) {
        word = s;
    }

    /**
     * Gibt das Wort zurck.
     *
     * @return Wort
     */
    public String getWord() {
        return word;
    }

    /**
     * Setzt die Wortart.
     *
     * @param t Wortart (TYPE_*)
     */
    public void setType(int t) {
        type = t;
    }

    /**
     * Gibt die Wortart zurck.
     *
     * @return Wortart (als TYPE_*)
     */
    public int getType() {
        return type;
    }

    /**
     * Setzt die Zusatzkategorie (tag)
     *
     * @param s Tag
     */
    public void setTag(String s) {
        tag = s;
    }

    /**
     * Gibt die Zusatzkategorie aus (tag)
     *
     * @return Tag
     */
    public String getTag() {
        return tag;
    }

    /**
     * Setzt die Bemerkung
     *
     * @param s Bemerkung
     */
    public void setRemark(String s) {
        remark = s;
    }

    /**
     * Gibt die Bemerkung zurck.
     *
     * @return Bemerkung
     */
    public String getRemark() {
        return remark;
    }

    /**
     * Gibt die StringList zurck, die die bersetzungen
     * speichert.
     *
     * @return StringList
     */
    public StringList getTranslations() {
        return translations;
    }

    /**
     * Gibt eine StringList zurck, die den XML-Code enthlt.
     *
     * @return XML-Code
     * @see Vocable#parseXML(String)
     */
    public StringList toXML() {
        StringList sl = new StringList();

        sl.add(INDENT + INDENT + "<Vocable>");
        sl.add(INDENT + INDENT + INDENT
               + "<Word type=\"" + TYPES[type] + "\" tag=\"" + tag + "\">"
               + word + "</Word>");
        for (int i = 0; i < translations.size(); i++) {
            sl.add(INDENT + INDENT + INDENT
                  + "<Translation>" + translations.get(i) + "</Translation>");
        }
        sl.add(INDENT + INDENT + INDENT
               + "<Remark>" + remark + "</Remark>");
        sl.add(INDENT + INDENT + "</Vocable>");

        return sl;
    }

    /**
     * Liest die Daten aus einem XML-Code aus. Davor wird das Objekt
     * neu initialisiert
     *
     * @param s String mit XML-Codes.
     * @see Vocable#parseXML(StringList)
     * @see Vocable#toXML
     */
    public void parseXML(String s) {
        int index;
        int length;
        String stemp;

        this.initialize();

        //* bis zum ersten "<Vocable>" lschen (inklusiv)
        index = s.indexOf("<Vocable>");
        length = "<Vocable>".length();
        s = s.substring(index + length);

        //* bis zum nchsten "<Word" lschen (inklusiv)
        index = s.indexOf("<Word");
        length = "<Word".length();
        s = s.substring(index + length);

        //* bis zum nchsten "type=\"" lschen (inklusiv)
        index = s.indexOf("type=\"");
        length = "type=\"".length();
        s = s.substring(index + length);

        //* bis zum nchsten "\"" = type (exklusiv)
        index = s.indexOf("\"");
        stemp = s.substring(0, index);

        if (stemp.equals("noun")) {
            setType(TYPE_NOUN);
        } else if (stemp.equals("verb")) {
            setType(TYPE_VERB);
        } else if (stemp.equals("adjective")) {
            setType(TYPE_ADJECTIVE);
        } else if (stemp.equals("pronoun")) {
            setType(TYPE_PRONOUN);
        } else if (stemp.equals("invariable")) {
            setType(TYPE_INVARIABLE);
        } else {
            setType(TYPE_OTHER);
        }

        s = s.substring(index + 1);

        //* bis zum nchsten "tag=\"" lschen (inklusiv)
        index = s.indexOf("tag=\"");
        length = "tag=\"".length();
        s = s.substring(index + length);

        //* bis zum nchsten "\"" = tag (exklusiv)
        index = s.indexOf("\"");
        stemp = s.substring(0, index);
        setTag(stemp);
        s = s.substring(index + 1);

        //* bis zum nchsten ">" lschen (inklusiv)
        index = s.indexOf(">");
        length = ">".length();
        s = s.substring(index + length);

        //* bis zum nchsten "</Word>" = word (exklusiv)
        index = s.indexOf("</Word>");
        stemp = s.substring(0, index);
        setWord(stemp);
        length = "</Word>".length();
        s = s.substring(index + length);

        //* solange bis index("<Translation>" = -1 &&
        //*             index("<Translation>" > index("<Remark>")
        while((s.indexOf("<Translation>") != -1) && (s.indexOf("<Translation>") < s.indexOf("<Remark>"))) {
            //* bis zum nchsten "<Translation>" lschen (inklusiv)
            index = s.indexOf("<Translation>");
            length = "<Translation>".length();
            s = s.substring(index + length);

            //* bis zum nchsten "</Translation>" = translations.add (exklusiv)
            index = s.indexOf("</Translation>");
            stemp = s.substring(0, index);
            getTranslations().add(stemp);
            length = "</Translation>".length();
            s = s.substring(index + length);
        }

        //* bis zum nchsten "<Remark>" lschen (inklusiv)
        index = s.indexOf("<Remark>");
        if (index != -1) {
            length = "<Remark>".length();
            s = s.substring(index + length);

            //* bis zum nchsten "</Remark>" = remark (exklusiv)
            index = s.indexOf("</Remark>");
            stemp = s.substring(0, index);
            setRemark(stemp);
        } else {
            setRemark("");
        }
    }

    /**
     * Liest die Daten aus einem XML-Code, der in einer StringList
     * gespeichert ist, aus.
     *
     * @param sl StringList mit XML-Codes.
     * @see Vocable#parseXML(String)
     */
    public void parseXML(StringList sl) {
        parseXML(sl.getText());
    }

    /**
     * Erstellt einen HTML-Code (exportiert die Daten).
     *
     * @return StringList mit HTML-Code.
     */
    public StringList toHTML() {
        StringList sl = new StringList();
        String s = "";

        sl.add(INDENT + "<tr>");
        sl.add(INDENT + INDENT + "<td>" + this.getWord() + "</td>");

        for (int i = 0; i < this.getTranslations().size(); i++) {
            s = s + this.getTranslations().get(i) + "; ";
        }
        if (this.getTranslations().size() > 0) {
            s = s.substring(0, s.length() - 2);
        }

        sl.add(INDENT + INDENT + "<td>" + s + "</td>");
        sl.add(INDENT + INDENT + "<td>" + this.getRemark() + "</td>");
        sl.add(INDENT + "</tr>");

        return sl;
    }

    /**
     * Gibt die Daten im Textformat aus.
     *
     * @return StringList
     */
    public StringList toText() {
        StringList sl = new StringList();
        String s;
        String tr = "";

        for (int i = 0; i < this.getTranslations().size(); i++) {
            tr = tr + this.getTranslations().get(i) + "; ";
        }
        tr = tr.substring(0, tr.length() - 2);

        s = this.getWord() + " | " + tr + " | " + this.getRemark();

        sl.add(s);
        return sl;
    }

    /**
     * Gibt die Daten (formatiert mit HTML) in einen String aus.
     * Wichtig fr JTree...
     */
    public String toString() {
        String s;
        s = "<html>";
        s = s + "<b>" + this.getWord() + "</b> | ";
        for (int i = 0; i < this.getTranslations().size(); i++) {
            s = s + this.getTranslations().get(i) + "; ";
        }
        /* letztes "; " entfernen */
        s = s.substring(0, s.length()-2);
        s = s + " | ";
        s = s + this.getRemark();
        s = s + "</html>";

        return s;
    }

    /**
     * Gibt ein DefaultMutableTreeNode-Objekt zurck.
     * Es kann keine Children haben...
     *
     * @see javax.swing.tree.DefaultMutableTreeNode
     */
    public DefaultMutableTreeNode getTreeNode() {
        return treeNode;
    }
}
