/*
 * Vocabulary.java     1.0    2001/12/27
 *
 * Copyright (c) 2001 Andreas Dangel <a.dangel@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package libraries;

import java.io.*;
import java.util.Vector;
import javax.swing.tree.*;

/**
 * Vocabulary reprsentiert eine komplette Lektionsdatei. Die Lektionsdatei
 * hat mehrere Lektionen, die ihrerseits wieder mehrere Vokabeln haben.
 *
 * @see Lesson
 * @see Vocable
 *
 * @version 1.0 2001/12/27
 * @author  Andreas Dangel <a.dangel@gmx.de>
 */
public class Vocabulary {

    /** Zeileneinschub beim XML-Code. */
    public static final String INDENT = "    ";

    private String title;
    private String description;
    private String date;
    private String author;
    private Vector lessons;

    private DefaultMutableTreeNode treeNode;

    /**
     * ruft initialize() auf.
     *
     * @see Vocabulary#initialize
     */
    public Vocabulary() {
        initialize();
    }

    /**
     * Initialisiert das Objekt. Stellt alle Variablen auf
     * null bzw. "".
     */
    public void initialize() {
        title = "";
        description = "";
        date = "";
        author = "";
        lessons = new Vector();

        createTreeNode();
    }

    /**
     * Erzeugt ein DefaultMutableTreeNode-Objekt und gibt dieses zurck.
     *
     * @return das TreeNode-Objekt
     */
    public DefaultMutableTreeNode createTreeNode() {
        treeNode = new DefaultMutableTreeNode(this);

        /* Lektionen */
        for (int i = 0; i < this.size(); i++) {
            treeNode.add(this.get(i).createTreeNode());
        }

        return treeNode;
    }


    /**
     * Setzt den Titel.
     *
     * @param s Titel
     */
    public void setTitle(String s) {
        title = s;
    }

    /**
     * Gibt den Titel zurck.
     *
     * @return Titel
     */
    public String getTitle() {
        return title;
    }

    /**
     * Setzt die Beschreibung.
     *
     * @param s Beschreibung
     */
    public void setDescription(String s) {
        description = s;
    }

    /**
     * Gibt die Beschreibung zurck.
     *
     * @return Beschreibung
     */
    public String getDescription() {
        return description;
    }

    /**
     * Setzt das Datums-Feld.
     *
     * @param s Datum-String.
     */
    public void setDate(String s) {
        date = s;
    }

    /**
     * Gibt das Datums-Feld zurck.
     *
     * @return Datum-String
     */
    public String getDate() {
        return date;
    }

    /**
     * Setzt das Autoren-Feld.
     *
     * @param s Autor
     */
    public void setAuthor(String s) {
        author = s;
    }

    /**
     * Gibt das Autoren-Feld zurck.
     *
     * @return Autor
     */
    public String getAuthor() {
        return author;
    }


    /**
     * Fgt eine Lektion an der gegebenen Position ein.
     *
     * @param index Position
     * @param les   Lektion
     */
    public void add(int index, Lesson les) {
        lessons.add(index, les);
    }

    /**
     * Fgt eine Lektion am Ende hinzu.
     *
     * @param les Lektion
     */
    public void add(Lesson les) {
        lessons.add(les);
    }

    /**
     * Lscht alle Lektionen.
     */
    public void clear() {
        lessons.clear();
    }

    /**
     * Gibt die Lektion an der gegebenen Position zurck.
     *
     * @param index Position
     * @return Lektion
     */
    public Lesson get(int index) {
        return (Lesson) lessons.get(index);
    }

    /**
     * Gibt den Index einer bestimmten Lektion zurck.
     *
     * @param les Lektion, die gesucht werden soll
     * @return den index oder -1, wenn die Lektion nicht vorhanden ist
     */
    public int indexOf(Lesson les) {
        return lessons.indexOf(les);
    }

    /**
     * Entfernt die Lektion an der gegebenen Position.
     *
     * @param index Position
     * @return die gelschte Lektion
     */
    public Lesson remove(int index) {
        return (Lesson) lessons.remove(index);
    }

    /**
     * Entfernt eine bestimmte Lektion.
     *
     * @param les Lektion, die gelscht werden soll.
     * @return true, wenn die Lektion gefunden und gelscht wurde.
     */
    public boolean remove(Lesson les) {
        int index = this.indexOf(les);
        if (index > -1) {
            this.remove(index);
            return true;
        }
        return false;
    }

    /**
     * Ersetzt die Lektion an der gegebenen Position.
     *
     * @param index Position
     * @param les die neue Lektion
     * @return die alte Lektion
     */
    public Lesson set(int index, Lesson les) {
        return (Lesson) lessons.set(index, les);
    }

    /**
     * Gibt die Anzahl der Lektionen zurck.
     *
     * @return Anzahl
     */
    public int size() {
        return lessons.size();
    }

    /**
     * Vertauscht zwei Lektionen in der Liste.
     *
     * @param index1 die erste Lektion
     * @param index2 die zweite Lektion
     */
    public void exchange(int index1, int index2) {
        if (index1 == index2) {
            // do nothing
        } else if (index1 < index2) {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 - 1));
        } else {
            this.add(index2, this.remove(index1));
            this.add(index1, this.remove(index2 + 1));
        }

    }

    /**
     * Fgt eine komplette Lektionsdatei hinzu.
     * Der Titel und so weiter wird nicht bernommen!
     *
     * @param vocab die Lektionsdatei
     */
    public void add(Vocabulary vocab) {
        for (int i = 0; i < vocab.size(); i++) {
            this.add(vocab.get(i));
        }
    }


    /**
     * Sucht eine besimmte Vokabel in der Lektionsdatei.
     *
     * @param voc Vokabel, die gesucht werden soll
     * @return eine Lektionsdatei mit Suchergebnissen
     */
    public Vocabulary search(Vocable voc) {
        Vocabulary vocab = new Vocabulary();
        Lesson les = new Lesson();

        vocab.setTitle("Results");
        les.setTitle("");

        for (int i = 0; i < this.size(); i++) {
            les.add(this.get(i).search(voc));
        }

        vocab.add(les);

        return vocab;
    }

    /**
     * Sucht eine Lektion mit dem angegebenen Titel und gibt diese zurck.
     *
     * @return die Lektion oder null wenn nichts gefunden wurde.
     */
    public Lesson searchLesson(String title) {
        Lesson ret = null;
    
        for (int i = 0; i < this.size(); i++) {
	    // ist es diese Lektion?
	    if (this.get(i).getTitle().equals(title)) {
	        ret = this.get(i);
		return ret;
	    }
	    // ist es eine der untergeordneten Lektionen dieser Lektion?
	    ret = this.get(i).searchLesson(title);
	    if (ret != null) {
	        return ret;
	    }
	}
    
        return null;
    }


    /**
     * Gibt des XML-Code zurck.
     *
     * @return StringList mit XML-Code.
     * @see Vocabulary#parseXML(String)
     */
    public StringList toXML() {
        StringList sl = new StringList();

        sl.add("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
        sl.add("<?xml-stylesheet href=\"vocabulary.xsl\" type=\"text/xsl\"?>");
        sl.add("<!DOCTYPE Vocabulary SYSTEM \"vocabulary.dtd\">");
        sl.add("");
        sl.add("<Vocabulary>");
        sl.add(INDENT + "<TitleOfVocabulary>" + this.getTitle()
                + "</TitleOfVocabulary>");
        sl.add(INDENT + "<Description>" + this.getDescription()
                + "</Description>");
        sl.add(INDENT + "<Date>" + this.getDate() + "</Date>");
        sl.add(INDENT + "<Author>" + this.getAuthor()
                + "</Author>");

        for (int i = 0; i < this.size(); i++) {
            sl.add("");
            sl.add(this.get(i).toXML());
        }

        sl.add("</Vocabulary>");

        return sl;
    }

    /**
     * Liest die Daten aus einem XML-Code aus.
     * Die Lektionsdatei wird zuvor neu initialisiert.
     *
     * @param s String mit XML-Code.
     * @see Vocabulary#toXML
     */
    public void parseXML(String s) {
        Lesson les;
        String stemp;
        int index;
        int length;

        initialize();

        /* alles bis zum ersten <Vocabulary> lschen (inklusiv) */
        index = s.indexOf("<Vocabulary>");
        length = "<Vocabulary>".length();
        s = s.substring(index + length);

        /* bis zum nchsten <TitleOfVocabulary> lschen (inklusiv) */
        index = s.indexOf("<TitleOfVocabulary>");
        length = "<TitleOfVocabulary>".length();
        s = s.substring(index + length);

        /* bis zum nchsten </TitleOf..> = title */
        index = s.indexOf("</TitleOfVocabulary>");
        length = "</TitleOfVocabulary>".length();
        stemp = s.substring(0, index);
        this.setTitle(stemp);
        s = s.substring(index + length);

        /* bis zum nchsten <Description> lschen (inklusiv) */
        index = s.indexOf("<Description>");
        length = "<Description>".length();
        s = s.substring(index + length);

        /* bis zum nchsten </Descrp..> = description */
        index = s.indexOf("</Description>");
        length = "</Description>".length();
        stemp = s.substring(0, index);
        this.setDescription(stemp);
        s = s.substring(index + length);

        /* bis zum nchsten <Date> lschen (inklusiv) */
        index = s.indexOf("<Date>");
        length = "<Date>".length();
        s = s.substring(index + length);

        /* bis zum nchsten </Date> = date */
        index = s.indexOf("</Date>");
        length = "</Date>".length();
        stemp = s.substring(0, index);
        this.setDate(stemp);
        s = s.substring(index + length);

        /* bis zum nchsten <Author> lschen (inklusiv) */
        index = s.indexOf("<Author>");
        length = "<Author>".length();
        s = s.substring(index + length);

        /* bis zum nchsten </Author> = author */
        index = s.indexOf("</Author>");
        length = "</Author>".length();
        stemp = s.substring(0, index);
        this.setAuthor(stemp);
        s = s.substring(index + length);


        /* untergeordnete Lektionen: */

        /* wiederhole solange wie Lektionen vorhanden sind */
        while (s.indexOf("<Lesson>") > -1) {
            /* erste Stufe */
            int depth = 1;

            /* sichere index */
            index = s.indexOf("<Lesson>");

            /* aktuelle Position */
            int position = index;

            /* widerhole solange depth > 0 */
            while (depth > 0) {
                if ((s.indexOf("<Lesson>", position + 1) > -1)
                        && (s.indexOf("<Lesson>", position + 1) < s.indexOf("</Lesson>", position + 1))) {
                    /* nchstes Element: <Lesson> */
                    depth++;

                    /* aktuelle Position */
                    position = s.indexOf("<Lesson>", position + 1);
                } else {
                    /* nchstes Element msste </Lesson> sein */
                    depth--;

                    /* aktuelle Position */
                    position = s.indexOf("</Lesson>", position + 1);
                }
            }

            /* jetzt msste depth == 0 sein */
            length = "</Lesson>".length();
            stemp = s.substring(index, position + length);
            s = s.substring(position + length);

            /* neue Lektion */
            les = new Lesson();
            les.parseXML(stemp);
            this.add(les);
        }
    }

    /**
     * Liest die Daten aus einem XML-Code aus.
     * Die Lektionsdatei wird zuvor neu initialisiert.
     *
     * @param sl StringList mit XML-Code.
     * @see Vocabulary#parseXML(String)
     */
    public void parseXML(StringList sl) {
        this.parseXML(sl.getText());
    }

    /**
     * Ldt eine Lektionsdatei aus einer Datei im XML-Format.
     * Die Lektionsdatei wird zuvor neu initialisiert.
     *
     * @param filename Dateiname
     */
    public void loadFromFile(String filename) {
        File file = new File(filename);
	if (!file.exists()) {
	    this.saveToFile(filename);
	}
	
        StringList sl = new StringList();
        sl.loadFromFile(filename);
        this.parseXML(sl);
    }

    /**
     * Speichert die Lektionsdatei in eine Datei im XML-Format.
     *
     * @param filename Dateiname
     */
    public void saveToFile(String filename) {
        this.toXML().saveToFile(filename);
    }

    /**
     * Gibt einen HTML-Code zurck (die Daten werden exportiert).
     *
     * @return StringList mit HTML-Code.
     */
    public StringList toHTML() {
        StringList sl = new StringList();

        sl.add("<html>");
        sl.add("<head>");
        sl.add(INDENT + "<title>" + this.getTitle() + "</title>");
        sl.add("</head>");
        sl.add("<body bgcolor=\"#FFFFFF\">");
        sl.add("<h1>" + this.getTitle() + "</h1>");
        sl.add("<dl>");
        sl.add(INDENT + "<dt>Description</dt>");
        sl.add(INDENT + INDENT + "<dd>" + this.getDescription() + "</dd>");
        sl.add(INDENT + "<dt>Date</dt>");
        sl.add(INDENT + INDENT + "<dd>" + this.getDate() + "</dd>");
        sl.add(INDENT + "<dt>Author</dt>");
        sl.add(INDENT + INDENT + "<dd>" + this.getAuthor() + "</dd>");
        sl.add("</dl>");

        for (int i = 0; i < this.size(); i++) {
            sl.add(this.get(i).toHTML());
        }

        sl.add("</body>");
        sl.add("</html>");

        return sl;
    }

    /**
     * Gibt die Lektionsdatei im Textformat aus (exportiert die Daten).
     *
     * @return StringList mit Textdaten
     */
    public StringList toText() {
        StringList sl = new StringList();
        String stemp = "";

        sl.add(this.getTitle());
        for (int i = 0; i < this.getTitle().length(); i++) {
            stemp = stemp + "^";
        }
        sl.add(stemp);
        sl.add("");
        sl.add("Description: " + this.getDescription());
        sl.add("Date       : " + this.getDate());
        sl.add("Author     : " + this.getAuthor());
        sl.add("");

        for (int i = 0; i < this.size(); i++) {
            sl.add(this.get(i).toText());
        }

        return sl;
    }

    /**
     * Gibt einen String (sogar mit HTML formatiert) mit dem Titel zurck.
     * Wichtig fr JTree....
     */
    public String toString() {
        String s;
        s = "<html>";
        s = s + "<b>" + this.getTitle() + "</b>";
        s = s + "</html>";

        return s;
    }

    /**
     * Gibt ein DefaultTreeModel-Objekt zurck, das in einem JTree-Objekt
     * benutzt werden kann.
     *
     * @see javax.swing.tree.DefaultTreeModel
     * @see javax.swing.JTree
     *
     * @return ein fertiges DefaultTreeModel-Objekt
     */
    public DefaultTreeModel getTreeModel() {
        createTreeNode();
        return new DefaultTreeModel(treeNode, true);
    }

    /**
     * Gibt ein DefaultMutableTreeNode-Objekt, das in einem JTree-Objekt
     * benutzt werden kann.
     *
     * @see javax.swing.tree.DefaultMutableTreeObjekt
     * @see javax.swing.JTree
     *
     * @return DefaultMutableTreeNode-Objekt
     */
    public DefaultMutableTreeNode getTreeNode() {
        return treeNode;
    }

    /**
     * Findet eine zu einer Vokabel die Lektion, in der die Vokabel ist.
     *
     * @param voc die Vokabel, die gesucht wird
     * @return die Lektion, wenn sie gefunden wurde; null, wenn nichts gefunden
     * wurde.
     */
    public Lesson find(Vocable voc) {
        Lesson les = null;

        for (int i = 0; i < this.size(); i++) {
            les = this.get(i).find(voc);
            if (les != null) {
                return les;
            }
        }

        return null;
    }
}
